diff mbox series

[PR,c++/91826] bogus error with alias namespace

Message ID aa388fe3-9fad-b560-204f-c88cae607a2c@acm.org
State New
Headers show
Series [PR,c++/91826] bogus error with alias namespace | expand

Commit Message

Nathan Sidwell Jan. 27, 2020, 2:35 p.m. UTC
I've committed this to fix 91826

My changes to is_nested_namespace broke is_ancestor's use where a 
namespace alias might be passed in.  This changes is_ancestor to look 
through the alias.

nathan
diff mbox series

Patch

 gcc/cp/ChangeLog                      |  5 +++++
 gcc/cp/name-lookup.c                  | 32 ++++++++++++++++++++------------
 gcc/testsuite/g++.dg/lookup/pr91826.C | 16 ++++++++++++++++
 3 files changed, 41 insertions(+), 12 deletions(-)

2020-01-27  Nathan Sidwell  <nathan@acm.org>

	PR c++/91826
	* name-lookup.c (is_ancestor): Allow CHILD to be a namespace alias.

diff --git c/gcc/testsuite/g++.dg/lookup/pr91826.C w/gcc/testsuite/g++.dg/lookup/pr91826.C
new file mode 100644
index 00000000000..2b313ece8a7
--- /dev/null
+++ w/gcc/testsuite/g++.dg/lookup/pr91826.C
@@ -0,0 +1,16 @@ 
+// PR 91826 bogus error with aliased namespace
+
+namespace N1 { class C1; }
+namespace A1 = N1;
+class A1::C1 {}; //Ok
+
+namespace N2
+{
+  namespace N { class C2; }
+  namespace A2 = N;
+  class A2::C2 {}; // { dg_bogus "does not enclose" }
+}
+
+namespace N3 { namespace N { class C3; } }
+namespace A3 = N3::N;
+class A3::C3 {}; //Ok
diff --git c/gcc/cp/ChangeLog w/gcc/cp/ChangeLog
index c01becefe87..32cc230fb71 100644
diff --git c/gcc/cp/name-lookup.c w/gcc/cp/name-lookup.c
index cd7a5816e46..129cfad9ad6 100644
--- c/gcc/cp/name-lookup.c
+++ w/gcc/cp/name-lookup.c
@@ -4012,38 +4012,46 @@  is_nested_namespace (tree ancestor, tree descendant, bool inline_only)
   return ancestor == descendant;
 }
 
-/* Returns true if ROOT (a namespace, class, or function) encloses
-   CHILD.  CHILD may be either a class type or a namespace.  */
+/* Returns true if ROOT (a non-alias namespace, class, or function)
+   encloses CHILD.  CHILD may be either a class type or a namespace
+   (maybe alias).  */
 
 bool
 is_ancestor (tree root, tree child)
 {
-  gcc_assert ((TREE_CODE (root) == NAMESPACE_DECL
-	       || TREE_CODE (root) == FUNCTION_DECL
-	       || CLASS_TYPE_P (root)));
-  gcc_assert ((TREE_CODE (child) == NAMESPACE_DECL
-	       || CLASS_TYPE_P (child)));
+  gcc_checking_assert ((TREE_CODE (root) == NAMESPACE_DECL
+			&& !DECL_NAMESPACE_ALIAS (root))
+		       || TREE_CODE (root) == FUNCTION_DECL
+		       || CLASS_TYPE_P (root));
+  gcc_checking_assert (TREE_CODE (child) == NAMESPACE_DECL
+		       || CLASS_TYPE_P (child));
 
-  /* The global namespace encloses everything.  */
+  /* The global namespace encloses everything.  Early-out for the
+     common case.  */
   if (root == global_namespace)
     return true;
 
-  /* Search until we reach namespace scope.  */
+  /* Search CHILD until we reach namespace scope.  */
   while (TREE_CODE (child) != NAMESPACE_DECL)
     {
       /* If we've reached the ROOT, it encloses CHILD.  */
       if (root == child)
 	return true;
+
       /* Go out one level.  */
       if (TYPE_P (child))
 	child = TYPE_NAME (child);
       child = CP_DECL_CONTEXT (child);
     }
 
-  if (TREE_CODE (root) == NAMESPACE_DECL)
-    return is_nested_namespace (root, child);
+  if (TREE_CODE (root) != NAMESPACE_DECL)
+    /* Failed to meet the non-namespace we were looking for.  */
+    return false;
+
+  if (tree alias = DECL_NAMESPACE_ALIAS (child))
+    child = alias;
 
-  return false;
+  return is_nested_namespace (root, child);
 }
 
 /* Enter the class or namespace scope indicated by T suitable for name